#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from onf_website.shared.casestudy import IWfCaseStudy
from onf_website.shared.service import IWfService
from onf_website.shared.view.skin.interfaces import IONFViewItemsPortletCarouselRendererSettings, \
    IONFViewItemsPortletNewsRendererSettings, IONFViewItemsPortletSummaryRendererSettings
from onf_website.skin.public import owl_carousel_init, owl_navigation
from pyams_content.shared.view.portlet.interfaces import IViewItemsPortletSettings
from pyams_default_theme.features.search.portlet import HIDDEN_HEADER_DISPLAY, START_HEADER_DISPLAY
from pyams_default_theme.shared.view.portlet import BaseViewItemsPortletRenderer
from pyams_i18n.interfaces import II18n
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_sequence.reference import get_reference_target
from pyams_skin.layer import IPyAMSLayer, IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.text import get_text_start


__docformat__ = 'restructuredtext'

from onf_website import _


#
# News list renderer
#

@factory_config(provided=IONFViewItemsPortletNewsRendererSettings)
class ONFViewItemsPortletNewsRendererSettings(Persistent, Contained):
    """View items portlet renderer settings"""

    list_reference = FieldProperty(IONFViewItemsPortletNewsRendererSettings['list_reference'])
    link_label = FieldProperty(IONFViewItemsPortletNewsRendererSettings['link_label'])

    def get_target(self):
        if self.list_reference is not None:
            return get_reference_target(self.list_reference)


@adapter_config(name='ONF::view.actus',
                context=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/actus.pt', layer=IPyAMSUserLayer)
class ONFViewItemsPortletNewsRenderer(BaseViewItemsPortletRenderer):
    """View items portlet renderer"""

    label = _("ONF: news and folders")
    weight = 11

    settings_interface = IONFViewItemsPortletNewsRendererSettings
    resources = (owl_carousel_init, owl_navigation)


@adapter_config(name='ONF::view.actus-tight',
                context=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/actus-tight.pt', layer=IPyAMSUserLayer)
class ONFTightViewItemsPortletNewsRenderer(BaseViewItemsPortletRenderer):
    """View items portlet tight renderer"""

    label = _("ONF: news and folder (tight version)")
    weight = 12

    settings_interface = IONFViewItemsPortletNewsRendererSettings


#
# Related contents renderer
#

@adapter_config(name='ONF::view.related',
                context=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/related.pt', layer=IPyAMSUserLayer)
class ONFViewRelatedItemsPortletRenderer(BaseViewItemsPortletRenderer):
    """Related items portlet renderer"""

    label = _("ONF: related contents")
    weight = 13

    resources = (owl_carousel_init, owl_navigation)


#
# Partners logos renderer
#

@adapter_config(name='ONF::view.partners',
                context=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/partners-logos.pt', layer=IPyAMSUserLayer)
class ONFViewPartnersLogosPortletRenderer(BaseViewItemsPortletRenderer):
    """Partners logos portlet renderer"""

    label = _("ONF: services partners logos")
    weight = 14

    @property
    def logos(self):
        settings = self.settings
        for content in settings.get_items(self.request):
            if not (IWfService.providedBy(content) or IWfCaseStudy.providedBy(content)):
                continue
            for logo in content.get_logos():
                if not logo.image:
                    continue
                yield content, logo


#
# Horizontal carousel renderer
#

@factory_config(provided=IONFViewItemsPortletCarouselRendererSettings)
class ONFViewItemsPortletCarouselRendererSettings(Persistent, Contained):
    """View items portlet renderer settings"""

    header = FieldProperty(IONFViewItemsPortletCarouselRendererSettings['header'])
    button_label = FieldProperty(IONFViewItemsPortletCarouselRendererSettings['button_label'])
    button_target = FieldProperty(IONFViewItemsPortletCarouselRendererSettings['button_target'])

    def get_target(self):
        if self.button_target is not None:
            return get_reference_target(self.button_target)


@adapter_config(name='ONF::view.carousel',
                context=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/carousel.pt', layer=IPyAMSUserLayer)
class ONFViewCarouselPortletRenderer(BaseViewItemsPortletRenderer):
    """View carousel portlet renderer"""

    label = _("ONF: horizontal carousel")
    weight = 15

    settings_interface = IONFViewItemsPortletCarouselRendererSettings


#
# Summary-like view items renderer
#

@factory_config(provided=IONFViewItemsPortletSummaryRendererSettings)
class ONFViewItemsPortletSummaryRendererSettings(Persistent, Contained):
    """View items portlet summary renderer settings"""

    header_display_mode = FieldProperty(
        IONFViewItemsPortletSummaryRendererSettings['header_display_mode'])
    start_length = FieldProperty(IONFViewItemsPortletSummaryRendererSettings['start_length'])
    button_label = FieldProperty(IONFViewItemsPortletSummaryRendererSettings['button_label'])


@adapter_config(name='ONF::view.summary',
                context=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/summary.pt', layer=IPyAMSUserLayer)
class ONFViewItemsPortletSummaryRenderer(BaseViewItemsPortletRenderer):
    """View items portlet summary renderer"""

    label = _("ONF: summary")
    weight = 16

    settings_interface = IONFViewItemsPortletSummaryRendererSettings

    def get_header(self, item):
        settings = self.renderer_settings
        display_mode = settings.header_display_mode
        if display_mode == HIDDEN_HEADER_DISPLAY:
            return ''
        header = II18n(item).query_attribute('header', request=self.request)
        if display_mode == START_HEADER_DISPLAY:
            header = get_text_start(header, settings.start_length)
        return header
